/*
 * emdadm - manage Linux "emd" devices aka RAID arrays.
 *
 * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
 *
 * Made this Assemble option as not supported.
 * Copyright (C) 2004 Adaptec Inc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Author: Neil Brown
 *    Email: <neilb@cse.unsw.edu.au>
 *    Paper: Neil Brown
 *           School of Computer Science and Engineering
 *           The University of New South Wales
 *           Sydney, 2052
 *           Australia
 */

#include "mdadm.h"

int Build(char *mddev, char *metaname, char *array_name, int chunk, int level, 
	  int raiddisks, mddev_dev_t devlist)
{
	/* Build a linear or raid0 arrays 
	 * We cannot really do any checks, we just do it.
	 * We call EMD_SET_ARRAY_INFO, EMD_ADD_NEW_DISK, EMD_RUN_ARRAY
	 */

	int i;
        int mdfd;
	struct stat stb;
	int subdevs = 0;
	mddev_dev_t dv;
	mdu_array_info_t array;
        unsigned long cookie;


        printf( "\nBuild is not supported by the emdadm utility ...\n" );
	printf( "The EMD driver does not support Raid devices without metadata.\n\n" );
	return 0;

	/* scan all devices, make sure they really are block devices */
	for (dv = devlist; dv; dv=dv->next) {
		if (stat(dv->devname, &stb)) {
			fprintf(stderr, Name ": Cannot find %s: %s\n",
				dv->devname, strerror(errno));
			return 1;
		}
		if ((stb.st_mode & S_IFMT) != S_IFBLK) {
			fprintf(stderr, Name ": %s is not a block device.\n",
				dv->devname);
			return 1;
		}
		subdevs++;
	}

	if (raiddisks != subdevs) {
		fprintf(stderr, Name ": requested %d devices in array but listed %d\n",
			raiddisks, subdevs);
		return 1;
	}

	strncpy(array.array_name, array_name, MAX_ARRAY_NAME_LENGTH);
	array.level = level;
	array.size = 0;
	array.nr_disks = raiddisks;
	array.raid_disks = raiddisks;

	array.md_minor = 0;
        mdfd = open( mddev, O_RDWR, 0);
	if (fstat(mdfd, &stb)==0)
		array.md_minor = MINOR(stb.st_rdev);
        close( mdfd );

	array.not_persistent = 1;
	array.state = 0;                /* not clean, but no errors */
	array.active_disks = raiddisks;
	array.working_disks = raiddisks;
	array.spare_disks = 0;
	array.failed_disks = 0;
	if (chunk == 0)  
		chunk = 64;
	array.chunk_size = chunk * 1024;
        memcpy(array.metaname, metaname, MD_METAPERS_NAME_LENGTH);
	if (ioctl(mdctl_fd, EMD_SET_ARRAY_INFO, &array)) {
		fprintf(stderr, Name ": EMD_SET_ARRAY_INFO failed for %s: %s\n",
			mddev, strerror(errno));
		return 1;
	}
	
	/* now add the devices */
	for ((i=0), (dv = devlist) ; dv ; i++, dv=dv->next) {
		mdu_disk_info_t disk;

		if (stat(dv->devname, &stb)) {
			fprintf(stderr, Name ": Wierd: %s has disappeared.\n",
				dv->devname);
			goto abort;
		}
		if ((stb.st_mode & S_IFMT)!= S_IFBLK) {
			fprintf(stderr, Name ": Wierd: %s is no longer a block device.\n",
				dv->devname);
			goto abort;
		}

		disk.number = i;
		disk.raid_disk = i;
		disk.state = 6;
		disk.major = MAJOR(stb.st_rdev);
		disk.minor = MINOR(stb.st_rdev);
                disk.parent_cookie = array.cookie;
		if (ioctl(mdctl_fd, EMD_ADD_NEW_DISK, &disk)) {
			fprintf(stderr, Name ": EMD_ADD_NEW_DISK failed for %s: %s\n",
				dv->devname, strerror(errno));
			goto abort;
		}

	}
	/* now to start it */

        cookie = array.cookie;
	if (ioctl(mdctl_fd, EMD_RUN_ARRAY, cookie)) {
		fprintf(stderr, Name ": EMD_RUN_ARRAY failed: %s\n",
			strerror(errno));
		goto abort;
	}

	fprintf(stderr, Name ": array %s built and started.\n",
		mddev);
	return 0;

 abort:

       ioctl(mdctl_fd, EMD_STOP_ARRAY, 0);

	return 1;
		
}

