/*
 * mdadm - manage Linux "md" devices aka RAID arrays.
 *
 * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
 *
 * Added pluggable metadata support.
 * Copyright (C) 2004 Adaptec Inc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Author: Neil Brown
 *    Email: <neilb@cse.unsw.edu.au>
 *    Paper: Neil Brown
 *           School of Computer Science and Engineering
 *           The University of New South Wales
 *           Sydney, 2052
 *           Australia
 */

#include	"mdadm.h"
#include	"dlink.h"

#if ! defined(__BIG_ENDIAN) && ! defined(__LITTLE_ENDIAN)
#error no endian defined
#endif
#include	"md_u.h"
#include	"md_p.h"
int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust)
{
	/* Read the raid meta array info from a device and
	 * display important content.
	 *
	 * If cannot be found, print reason
	 *
	 * Print:
	 *   version, ctime, level, size, raid+spare+
	 *   prefered minor
	 *   utime, state etc
	 *
	 * If (brief) gather devices for same array and just print a mdadm.conf line including devices=
	 * if devlist==NULL, use conf_get_devs(
	 */
        mdu_array_info_t array;
	int rv = 0;
	int err;
	int spares = 0;

	struct array {
		mdu_array_info_t array;
		void *devs;
		struct array *next;
	} *arrays = NULL;

	for (; devlist ; devlist=devlist->next) {

		err = get_array_info(devlist->devname, &array);
	        if (err != 0) {
                        fprintf(stderr, Name ": cannot get array detail for %s: %s\n",
 				devlist->devname, strerror(errno));
	        }

        	if (err && (brief||scan))
			continue;

		if (err) {
			rv = 1;
			continue;
		}
		    
		/* Ok, its good enough to try, though the checksum could be wrong */
		if (brief) {
			struct array *ap;
			char *d;
			for (ap=arrays; ap; ap=ap->next) {
				if (compare_array(&ap->array, &array)==0)
					break;
			}
			if (!ap) {
				ap = malloc(sizeof(*ap));
				ap->array = array;
				ap->devs = dl_head();
				ap->next = arrays;
				arrays = ap;
			}
			d = dl_strdup(devlist->devname);
			dl_add(ap->devs, d);
		} else {
			print_array_info(devlist->devname, &array);
		        rv = print_disk_info(&array, brief, 0, devlist->devname);
		}

	}
	
	if (brief) {
		struct array *ap;
		for (ap = arrays; ap; ap = ap->next) {
			char sep = '=';
			char *c  = map_num(pers, ap->array.level);
			int i;
			int rc;
			mdu_disk_info_t disk_info;
			char *devname;
			
			printf("ARRAY /dev/md%d level=%s num-devices=%d",
			       ap->array.md_minor, c?c:"-unknown-", ap->array.raid_disks);
			if (spares) printf(" spares=%d", spares);
			printf("\n   devices ");
			for (i = 0; i<(signed int)(array.raid_disks+array.spare_disks); i++) {
				rc = get_disk_info_indexed(array.cookie, i, &disk_info);
				if (rc == 0) {
					devname = map_dev(disk_info.major, disk_info.minor);
					printf("%c %s", sep, devname);
					sep=',';
				}
			}
			printf("\n");
		}
	}
	return rv;
}
