/*
 * emdadm - manage Linux "emd" devices aka RAID arrays.
 *
 * Copyright (C) 2002 Neil Brown <neilb@cse.unsw.edu.au>
 *
 * Added pluggable metadata support.
 * Copyright (C) 2004 Adaptec Inc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Author: Neil Brown
 *    Email: <neilb@cse.unsw.edu.au>
 *    Paper: Neil Brown
 *           School of Computer Science and Engineering
 *           The University of New South Wales
 *           Sydney, 2052
 *           Australia
 */

#include	"mdadm.h"
#include	"md_p.h"
#include	"md_u.h"


int Query(char *dev)
{
	/* Give a brief description of the device and
	 * whether it is an emd device 
	 */
        int fd = open(dev, O_RDONLY, 0); 
	int ioctlerr;
	mdu_array_info_t array;
	unsigned long long larray_size;
	unsigned long array_size;
	struct stat stb;
	mdu_disk_info_t disc;

	if (fd < 0){
		fprintf(stderr, Name ": cannot open %s: %s\n",
			dev, strerror(errno));
		return 1;
	}

	ioctlerr = get_array_info( dev, &array );
	fstat(fd, &stb); 

	if (ioctlerr == 0) {
#ifdef BLKGETSIZE64
		if (ioctl(fd, BLKGETSIZE64, &larray_size)==0)
			;
		else
#endif
			if (ioctl(fd, BLKGETSIZE, &array_size)==0) {
				larray_size = array_size;
				larray_size <<= 9;
			} else larray_size = 0;

		if (MAJOR(stb.st_rdev) != MDP_MAJOR) 
    			printf("%s: is an emd device\n", dev);
		else
    			printf("%s: is an emd array\n", dev);
	}
        else { 
		printf("%s: is not part of any raid array\n", dev);
		close(fd);
                return 1;
        }

	close(fd);

	printf("%s: %s %s %d devices, %d spare%s. Use emdadm --detail for more detail.\n",
		       dev,
		       mb_size(larray_size),
		       map_num(pers, array.level),
		       array.raid_disks,
		       array.spare_disks, array.spare_disks==1?"":"s");
		
	if (MAJOR(stb.st_rdev) != MDP_MAJOR) {
		get_disk_info(dev, &disc);
		printf("%s: device %d in %d devices is active in %s md%d. Use emdadm --examine for more detail.\n",
			dev, 
			disc.number,
	                array.raid_disks,
			map_num(pers, array.level),
			array.md_minor);
	}

	return 0;
}

	
