/*
   md_u.h : user <=> kernel API between Linux raidtools and EMD RAID drivers
          Copyright (C) 1998 Ingo Molnar

   Pluggable metadata support.
   	  Copyright (C) 2004 Adaptec Inc.
	  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
   
   You should have received a copy of the GNU General Public License
   (for example /usr/src/linux/COPYING); if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#ifndef _MD_U_H
#define _MD_U_H

#define EMD_MAJOR	153
#define EMD_MINOR_SHIFT	4
#define EMDCTL_MINOR	202
#define EMDMINOR(mddev)	((mddev)->unit << EMD_MINOR_SHIFT)

#define MD_METAPERS_NAME_LENGTH 32
#define MAX_ARRAY_NAME_LENGTH 16
/* ioctls */

/* status */
#define RAID_VERSION		_IOR (EMD_MAJOR, 0x10, mdu_version_t)
#define GET_ARRAY_INFO		_IOR (EMD_MAJOR, 0x11, mdu_array_info_t)
#define GET_DISK_INFO		_IOR (EMD_MAJOR, 0x12, mdu_disk_info_t)
#define PRINT_RAID_DEBUG	_IO (EMD_MAJOR, 0x13)
#define RAID_AUTORUN		_IO (EMD_MAJOR, 0x14)

/* configuration */
#define CLEAR_ARRAY		_IO (EMD_MAJOR, 0x20)
#define ADD_NEW_DISK		_IOW (EMD_MAJOR, 0x21, mdu_disk_info_t)
#define HOT_REMOVE_DISK		_IO (EMD_MAJOR, 0x22)
#define SET_ARRAY_INFO		_IOW (EMD_MAJOR, 0x23, mdu_array_info_t)
#define SET_DISK_INFO		_IO (EMD_MAJOR, 0x24)
#define WRITE_RAID_INFO		_IO (EMD_MAJOR, 0x25)
#define UNPROTECT_ARRAY		_IO (EMD_MAJOR, 0x26)
#define PROTECT_ARRAY		_IO (EMD_MAJOR, 0x27)
#define HOT_ADD_DISK		_IO (EMD_MAJOR, 0x28)
#define SET_DISK_FAULTY		_IO (EMD_MAJOR, 0x29)
#define HOT_GENERATE_ERROR	_IO (EMD_MAJOR, 0x2a)

/* usage */
#define RUN_ARRAY		_IOW (EMD_MAJOR, 0x30, mdu_param_t)
#define START_ARRAY		_IO (EMD_MAJOR, 0x31)
#define STOP_ARRAY		_IO (EMD_MAJOR, 0x32)
#define STOP_ARRAY_RO		_IO (EMD_MAJOR, 0x33)
#define RESTART_ARRAY_RW	_IO (EMD_MAJOR, 0x34)

/*******************/
/* New EMD IOCTL's */
/*******************/

#define EMD_GET_DEVICE_INFO	      _IOW (EMD_MAJOR, 0x40, mdu_cookie_info_t)
#define EMD_GET_MEMBER_INFO	      _IOW (EMD_MAJOR, 0x41, mdu_cookie_info_t)
#define EMD_GET_DISK_INFO	      _IOW (EMD_MAJOR, 0x42, mdu_disk_info_t)
#define EMD_GET_DISK_INFO_FROM_PARENT _IOW (EMD_MAJOR, 0x43, mdu_disk_info_t)
#define EMD_GET_ARRAY_INFO	      _IOW (EMD_MAJOR, 0x44, mdu_array_info_t)
#define EMD_KILL_META_DATA	      _IOW (EMD_MAJOR, 0x45, mdu_disk_info_t)

#define EMD_GET_TASK_STATUS_INFO      _IOW (EMD_MAJOR, 0x46, unsigned long)

#define EMD_CLEAR_ARRAY		      _IOW (EMD_MAJOR, 0x47, unsigned long)
#define EMD_HOT_REMOVE_DISK	      _IOW (EMD_MAJOR, 0x48, mdu_disk_info_t)
#define EMD_SET_DISK_INFO	      _IOW (EMD_MAJOR, 0x49, mdu_disk_info_t)
#define EMD_WRITE_RAID_INFO	      _IOW (EMD_MAJOR, 0x4a, unsigned short)
#define EMD_UNPROTECT_ARRAY	      _IOW (EMD_MAJOR, 0x4b, unsigned long)
#define EMD_PROTECT_ARRAY	      _IOW (EMD_MAJOR, 0x4c, unsigned long)
#define EMD_HOT_ADD_DISK	      _IOW (EMD_MAJOR, 0x4d, mdu_disk_info_t)
#define EMD_SET_DISK_FAULTY	      _IOW (EMD_MAJOR, 0x4e, mdu_disk_info_t)

#define EMD_SET_ARRAY_INFO	      _IOW (EMD_MAJOR, 0x4f, mdu_array_info_t)
#define EMD_ADD_NEW_DISK	      _IOW (EMD_MAJOR, 0x50, mdu_disk_info_t)

#define EMD_RUN_ARRAY		      _IOW (EMD_MAJOR, 0x51, mdu_param_t)
#define EMD_START_ARRAY		      _IOW (EMD_MAJOR, 0x52, unsigned long)
#define EMD_STOP_ARRAY		      _IOW (EMD_MAJOR, 0x53, unsigned long)
#define EMD_STOP_ARRAY_RO	      _IOW (EMD_MAJOR, 0x54, mdu_major_minor_t)
#define EMD_RESTART_ARRAY_RW	      _IOW (EMD_MAJOR, 0x55, mdu_major_minor_t)
#define EMD_GET_FREESIZE	      _IOW (EMD_MAJOR, 0x56, mdu_dev_t) 
#define EMD_GET_NUM_OF_METAPERS	      _IOW (EMD_MAJOR, 0x57, int)
#define EMD_GET_METAPERS	      _IOW (EMD_MAJOR, 0x58, unsigned long)

/*
 * The states are defined as independant bits
 * to facilitate checking for multiple states
 * with a simple mask.
 */
enum mdu_mstate_e
{
	MDU_MSTATE_UNKNOWN	  = 0x00,
	MDU_MSTATE_OPTIMAL	  = 0x01,
	MDU_MSTATE_DEGRADED	  = 0x02,
	MDU_MSTATE_FAILED	  = 0x04,/* No increments to refcount allowed.*/
	MDU_MSTATE_TEARDOWN	  = 0x08,
	MDU_MSTATE_MODIFIER_MASK  = 0xF0,
	MDU_MSTATE_REBUILD_TARGET = 0x10,
	MDU_MSTATE_INITIALIZING	  = 0x20
};
typedef enum mdu_mstate_e mdu_mstate_t;

typedef struct mdu_version_s {
	int major;
	int minor;
	int patchlevel;
} mdu_version_t;

typedef struct mdu_major_minor_s {
	unsigned int major;
	unsigned int minor;
} mdu_major_minor_t;

typedef struct mdu_device_metapers_s {
	mdu_major_minor_t major_minor;
	char metapers_name[MD_METAPERS_NAME_LENGTH];
} mdu_device_metapers_t;

typedef union mdu_dev_u
{
	mdu_device_metapers_t device_metapers;
	unsigned long freesize;
} mdu_dev_t;

typedef struct mdu_array_info_s {
	/*
	 * Generic constant information
	 */
	int major_version;
	int minor_version;
	int patch_version;
	int ctime;
	int level;
	int size;
	int nr_disks;
	int raid_disks;
	int md_minor;
	int not_persistent;

	/*
	 * Generic state information
	 */
	int utime;		/*  0 Superblock update time		      */
	mdu_mstate_t state;	/*  1 State bits (degraded, optimal, ...)     */
	int active_disks;	/*  2 Number of currently active disks	      */
	int working_disks;	/*  3 Number of working disks		      */
	int failed_disks;	/*  4 Number of failed disks		      */
	int spare_disks;	/*  5 Number of spare disks		      */

	/*
	 * Personality information
	 */
	int layout;		/*  0 the array's physical layout	      */
	int chunk_size;	/*  1 chunk size in bytes		      */

	/*
	 * EMD MetaData Info 
	 */  
	//mdu_metatypes_t metadata_type;
	unsigned char metaname[MD_METAPERS_NAME_LENGTH];
	unsigned long parent_cookie;            
	unsigned long cookie;
	char array_name[MAX_ARRAY_NAME_LENGTH];

} mdu_array_info_t;


enum mdu_device_type_e
{
	MDU_PHYSICAL_DEVICE,
	MDU_HOT_SPARE_DEVICE,
	MDU_ARRAY,
	MDU_HIDDEN_ARRAY
};
typedef enum mdu_device_type_e mdu_device_type_t;


typedef struct mdu_disk_info_s {
	/*
	 * configuration/status of one particular disk
	 */
	int number;
	int major;
	int minor;
	int raid_disk;
	mdu_mstate_t state;
	unsigned long parent_cookie;            
	unsigned long cookie;    
	mdu_device_type_t device_type; 
} mdu_disk_info_t;


typedef struct mdu_start_info_s {
	/*
	 * configuration/status of one particular disk
	 */
	int major;
	int minor;
	int raid_disk;
	int state;

} mdu_start_info_t;


typedef struct mdu_param_s
{
	int			personality;	/* 1,2,3,4 */
	int			chunk_size;	/* in bytes */
	int			max_fault;	/* unused for now */
} mdu_param_t;

typedef struct mdu_cookie_info_s {
	/*
	 * cookie info of one device or array
	 */
	mdu_major_minor_t major_minor;
	unsigned long	parent_cookie;            
	unsigned long	cookie;                  
	int		type;
	int		subtype; 
	unsigned long	gencount;

} mdu_cookie_info_t;
#endif 

